/** Programm zur Verwaltung von Personen
  * @author Vitus Lorenz-Meyer
  * @version 2.0 vom 26.5.2002:
  	- Elemente nun in binrem Suchbaum
  	- Suchbaum stellt accept fr visitor Interface zur Verfgung (In-,Pre-,Post- und LevelOrder)
  	- BTVisitor Interface definiert 3 Methoden zum visiten eines Baumes, wird immer mit der aktuellen Node aufgerufen, start,end und visitNode
  	- I/O Implementiert
        - serialisiert formatiert nach einer Datei 
        - liest Datenstze aus einer Datei getrennt durch #
        - AdressBuchException hinzugefgt, die Fehler anzeigt und abgefangen werden kann
        - Felder fr Telefonnummer und Hausnummernzusatz zu Person hinzugefgt
        - htmlString() zu jeder Klasse hinzugefgt, gibt einen String formatiert fr HTML Ausgabe
  	
  * history:
  -----------
      version 1.5 vom 1.5 2002:
  	- Klasse Person erweitert um Strae,Hausnummer,PLZ und Ort
  	- Person wirft eine IllegalArgumentException, falls ein Parameter falsch
  	- toString liefert eine Person nun formatiert wie eine Adresse
  	- Elemente nun in einer sortierten linearen Liste gespeichert
  	- Lnge der Liste,sowie Kopf- und Schwanzzeiger in Personen aufgenommen
  	- personAusgeben() liefert alle Personen mit dem gleichen Nachnamen
  	- Interface Adressbuch definiert, wird von Personen implemenetiert
      version 1.0 vom 18.4.2002:
	- erste Version, Person enthlt nur Vor- und Nachname
	- Elemente werden in einer linear, einfach verketteten Liste gespeichert
	- implementiert toString, sowie eine Suche
  */
/**ein Binrer Suchbaum */
class BinaryTree{

	public Object element,item;
	public int high;
	public BinaryTree right,left;



	public BinaryTree(){
		left=right=null;
		element=item=null;
		high=-2;
	}
	public BinaryTree(Object i, Object e){
		left=new BinaryTree();
		right=new BinaryTree();
		item=i;
		element=e;
		high=-1;
	}
	public boolean isEmpty(){
		return (item==null);
	}
	
	public int insert(Object i,Object e){
		if (item==null){
			item=i;
			element=e;
			high=0;
			left=new BinaryTree();
			right=new BinaryTree();
			
		}
		
		else{
			int p=0;
			if (((Comparable)item).compareTo(i) > 0) p=left.insert(i,e);
			else if (((Comparable)item).compareTo(i) < 0) p=right.insert(i,e);
			else{
				throw new RuntimeException("BT.insert: Element bereits vorhanden");
				
			}
			if ((p+1)>high) high=p+1;
		}
		return high;
	}
	
	public BinaryTree delete(Object i){
		if (item.equals(i)){
			if (left.isEmpty()){
				if (right.isEmpty()) return new BinaryTree();
				else return right;
			}else{	if (right.isEmpty()) return left;
				else{	BinaryTree a = right.min();
					a.right = right.delete(a.item);
					a.left=left;
					return a;
				}
			}
		}else{
			if (item==null) throw new RuntimeException("BT.delete: Element nicht im Baum vorhanden");
			else if (((Comparable)item).compareTo(i) > 0) left = left.delete(i);
			else right = right.delete(i);
		}
		return this;
  	}
	public BinaryTree min(){
		BinaryTree x = this;
		while (!x.left.isEmpty()) x=x.left;
		return x;
	}	
	public BinaryTree max(){
		BinaryTree x = this;
		while (!x.right.isEmpty()) x=x.right;
		return x;
	}
	public int length(){
		if (item==null) return 0;
		else return (1+left.length()+right.length());
	}
	public boolean isIn(Object i){
		if (item==null) return false;
		else if (item.equals(i)) return true;
		else if (((Comparable)item).compareTo(i) > 0) return left.isIn(i);
		else return right.isIn(i);
		
	}
	
	public void acceptIn(BTVisitor visitor){
		if (item!=null){
			left.acceptIn(visitor);
			visitor.visitBTNode(this);
			right.acceptIn(visitor);
		}
	}
	public void acceptLevel(BTVisitor visitor,int level){
		//if (item==null) visitor.BTLeaf(high-level);
		if (level==0) visitor.visitBTNode(this);
		else {	
			
			left.acceptLevel(visitor,level-1);
			right.acceptLevel(visitor,level-1);
			
		}
		
	}
	public void acceptPre(BTVisitor visitor){
		if (item!=null){
			visitor.visitBTNode(this);
			left.acceptPre(visitor);
			right.acceptPre(visitor);
		}
	}
	public void acceptPost(BTVisitor visitor){
		if (item!=null){
			left.acceptPost(visitor);
			right.acceptPost(visitor);
			visitor.visitBTNode(this);
		}
	}
	public String toString(){
		if (item==null) return "";
		else return (left.toString() + element.toString()+"\r\n"+ right.toString());
	}
	public String htmlString(){
		if (item==null) return "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
		else return "<center><table width="+(high==0 ? 150:((int)(Math.pow(2,high)*(150/Math.pow(1.5,high-1)))))+" border=1 cellpadding=5 cellspacing=5>\r\n<tr><td colspan=2 align=center valign=top>"+(((Person_voll)(element)).htmlString())+"</td></tr>\r\n<tr><td align=center valign=top width=50%>"
			+left.htmlString()+"</td><td align=center valign=top width=50%>"+ right.htmlString()+"</td></tr></table></center>\r\n";
	}
}