//Haupt GUI File
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** Menu zum Hauptfenster
  * @version 1.0
  * @author Markus Burger-Scheidlin 2683838
  *         Vitus Lorenz-Meyer 2691370
  */
class MainMenu extends MenuBar
{
  /** fgt ein Menu Item hinzu
  * @param menu das Menu, zu dem hinzugefgt werden soll
  * @param name der Name des Entrys und auch der Command
  * @param vkey, der key fr den shortcut
  * @param listener die KLasse, die bei einem Event aufgerufen werden soll (impl. ActionListener)
  */
	private static void addNewMenuItem(Menu menu, String name, int vkey, ActionListener listener)
  {
	MenuShortcut shortcut = null;
	MenuItem mi;
	if (vkey!=0) shortcut=new MenuShortcut(vkey);
	if (shortcut != null) {
		mi = new MenuItem(name, shortcut);
	} else {
		mi = new MenuItem(name);
	}
	mi.setActionCommand(name);
	mi.addActionListener(listener);
	menu.add(mi);
  }
  /** Konstruktor
  * @param listener die KLasse, die bei einem Event aufgerufen werden soll (impl. ActionListener)
  */
  public MainMenu(ActionListener listener)
  {
	Menu m;
	m = new Menu("Datei");
	
	addNewMenuItem(m, "Datei ffnen",KeyEvent.VK_O, listener);
	addNewMenuItem(m, "Wrterbuch ffnen",KeyEvent.VK_W,  listener);
	m.addSeparator();
	addNewMenuItem(m, "Datei speichern",KeyEvent.VK_S,  listener);
	addNewMenuItem(m, "Wrterbuch speichern",KeyEvent.VK_Q,  listener);
	addNewMenuItem(m, "Beenden",KeyEvent.VK_E, listener);
	add(m);

	m = new Menu("gogogo!");
	addNewMenuItem(m, "correct",KeyEvent.VK_C,  listener);
	addNewMenuItem(m, "show Wrterbuch",KeyEvent.VK_G,  listener);
	addNewMenuItem(m, "show neverTouch-Wrter",KeyEvent.VK_H,  listener);
	add(m);
  }
}

/** Haupt GUI Klasse zum Wrterbuch checka
  * @version 1.0
  * @author Markus Burger-Scheidlin 2683838
  *         Vitus Lorenz-Meyer 2691370
  */
public class WBuchChecka extends Frame implements ActionListener
{
  /** die Statuszeile am unteren Rand
	*/
  private Label status;
  
  /** Haupttext Area
	*/
  public  TextArea texta;
  
  /** der Text, um dens hier geht ;)
	*/
  private String text;
  
  /** grade betrachtetes Wort
	*/
  private String cur="";
  
  /** das Wrterbuch
	*/
  private wbuch dasBuch;
  
  /** Wrterbuch nicht gespeichert...
	*/
  private boolean Wunsaved;
  
  /** File nicht gespeichert...
	*/
  private boolean Funsaved;
  
  /** der Correctur-Dialog
	*/
  private CorrectDialog Cd;
  
  /** Tokenizer zum trennen des Textes
	*/
  private StringTokenizer strt;
  
  /** EVKL fr Wrter, die nicht ins Wrterbuch sollen, aber auch nciht noch mal gefragt werden
	*/
  Vector  WordsNeverChange;
  
  /** Postionsmerker
	*/
  int pos,oldsize;
  
  /** System separator fr Zeilen
	*/
  private String sep;
  
  /** main-Methode
   * @param Argumente
  */
  public static void main(String[] args)
  {
  	WBuchChecka wnd = new WBuchChecka();
  }

  /** Konstruktor
   * 
  */
  public WBuchChecka()
  {	//Konstruktor von Frame
	super("WordChecka v1");
	setLocation(150,150);
	setSize(300,200);
	setMenuBar(new MainMenu(this));
	setLayout(new BorderLayout());    
	
	//listener frs Schlieen des Fensters
	addWindowListener(new WindowClosingAdapter(true));
	add("Center", texta=new TextArea());
	add("South", status=new Label("ready",Label.LEFT));
	text="";
	Wunsaved=Funsaved=false;
	        
	sep=System.getProperty("line.separator");
	dasBuch = new Woerterbuch();
	pack(); 
	setVisible(true);
	
  }
  /** Hilfsmethode, die das nchste zu checkende Wort sucht
   * 
  */  
  private void nextWordCheck(){
		//solange noch Wrter vorhanden sind und
		//der Correction-Dialog nicht beendet wurde und
		//das CD fenster nicht sichtbar ist, sind zwar Wrter vorhanden, aber sie sind schon im WB enthalten
		//wenn aber das CD sichtbar ist, wird auf eine Eingabe vom Benutzer gewartet
		//dann darf die Schleife nicht weiterlaufen... sie wird beim nchsten Aufruf weiterlaufen
		while (strt.hasMoreTokens()&&Cd.running&&!(Cd.isVisible())){
			//Wort nicht im WB und auch nicht in den nicht mehr zu betrachtenden Wrtern?
                	if (!WordsNeverChange.contains(cur=strt.nextToken())&&(!dasBuch.checkWord(cur)) ){
                		
                		//index des akt. Wortes im Text
                		pos=text.indexOf(cur);
                		
                		//alte Gre... fr delta(Text) wichtig
                		oldsize=text.length();
                		//select das akt. Wort im Text fr doofen Benutzer
                		texta.select(pos,pos+cur.length());
                		
                		//status.setText("correcting \""+cur+"\"");
                		//Titel des CD setzen
                		Cd.setTitle("correct "+cur);
                		//und CD visible machen
                		Cd.setVisible(true);           		
                	}
		}
		if (!Cd.running) status.setText("correcting abgebrochen...");
		else if (!strt.hasMoreTokens()) status.setText("correcting beendet");
			
  }
  
  /** wird aufgerugen, wenn was im Fenster passiert ist
  * @param event ist ein Event Object, das besagt, was passiert ist
  */
  public void actionPerformed(ActionEvent event)
  {
  	String filename;
	String cmd = event.getActionCommand();
	String re="";
	FileDialog a;
	
	    
	if (cmd.equals("Datei ffnen")){
	try{	
		String line;
		int i=0;
		text="";
		a = new FileDialog(this,"Datei ffnen",FileDialog.LOAD);
		a.show();
		if ((filename=a.getFile())!=null){
			BufferedReader f = new BufferedReader(
           			  new FileReader(filename));
       			while ((line = f.readLine()) != null){
				i++;
				text+=line+sep;
			}
			texta.setText(text);
			status.setText("Datei "+filename+" geffnet, "+i+" Zeilen gelesen");
			Funsaved=false;
		}
	}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim ffnen der Datei:   "+e,this);}
	
	}else if (cmd.equals("Wrterbuch ffnen")){
	try{
		a = new FileDialog(this,"WrterBuch ffnen",FileDialog.LOAD);
		a.show();
		if ((filename=a.getFile())!=null){
			int read=dasBuch.readFromFile(filename);
			status.setText("Wrterbuch "+filename+" geffnet, "+read+" Wrter gelesen");
			Wunsaved=false;
			
		}
	}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim ffnen des Wrterbuchs:   "+e,this);}
	
	}else if (cmd.equals("Datei speichern")){
	try{
		a = new FileDialog(this,"Text speichern",FileDialog.SAVE);
		a.show();
		if ((filename=a.getFile())!=null){
		
			FileWriter ausgabe = new FileWriter(filename, true);
			ausgabe.write(text);
			ausgabe.close();
			status.setText("Datei "+filename+" gespeichert");
			Funsaved=false;
		}
	}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim Schreiben der Datei:   "+e,this);}
			
	}else if (cmd.equals("Wrterbuch speichern")){
	try{	
		a = new FileDialog(this,"Wrterbuch speichern",FileDialog.LOAD);
		a.show();
		if ((filename=a.getFile())!=null){
			dasBuch.writeToFile(filename);
			status.setText("Wrterbuch in "+filename+" geschrieben");
			Wunsaved=false;
		}
	}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim Schreiben der Datei:   "+e,this);}
	
	}else if (cmd.equals("correct")){
	try{	
		
                Cd = new CorrectDialog(this);
                strt = new StringTokenizer(text,"\r\n\t\f .,;:-_\"#'+*^!$%&~@/\\(){}=?`",false);
                WordsNeverChange = new Vector();
                cur="";re="";pos=oldsize=0;
        	
		nextWordCheck();
   	
   	}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim correcten der Datei:   "+e,this);}
	//correct: korrigiertes oder altes Wort wird geholt und wenn nicht vorhanden, geaddet
	}else if (cmd.equals("correct word")){
	try{
		Cd.setVisible(false);	//corecctDialog verstecken
		
		//substring von aktueller pos, delta TextLnge + Lnge des betrachteten Wortes (+pos, weil substring "int end" erwartet)
        	text=texta.getText();
        	re=text.substring(pos,(text.length()-oldsize)+cur.length()+pos);
        	if (text.length()!=oldsize) Funsaved=true;
        	if (!dasBuch.checkWord(re)){
        		 dasBuch.addWord(re);
        		 status.setText("word "+re+" added");
        		 Wunsaved=true;
        	        }
                
		nextWordCheck();
		
   	}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim correcten der Datei:   "+e,this);}
	//next: Wort soll nicht ins Wrterbuch, dh. es soll auch nicht noch 1x nachgefragt werden
   	}else if (cmd.equals("next")){
	try{
		Cd.setVisible(false);
                WordsNeverChange.addElement(cur);
                
                
		nextWordCheck();
   	}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim correcten der Datei:   "+e,this);}
	}else if (cmd.equals("show Wrterbuch")){
	try{
		new MessageBox("Wrterbuch",dasBuch.toString(),this);
   	}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim correcten der Datei:   "+e,this);}
	
	}else if (cmd.equals("show neverTouch-Wrter")){
	try{
		new MessageBox("Wrterbuch",WordsNeverChange.toString(),this);
   	}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim correcten der Datei:   "+e,this);}
		
	
	
	}else if (cmd.equals("Beenden")){
		if (Wunsaved || Funsaved){
			MessageBox b=new MessageBox(this);
			switch (b.result){
			case 1:
				return;
			case 2:
				if (Funsaved){
				try{
					a = new FileDialog(this,"Text speichern",FileDialog.SAVE);
					a.show();
					if ((filename=a.getFile())!=null){
			
						FileWriter ausgabe = new FileWriter(filename, true);
						ausgabe.write(text);
						ausgabe.close();
						status.setText("Datei "+filename+" gespeichert");
					}
				}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim Schreiben der Datei:   "+e,this);}
				}if (Wunsaved){
				try{	
					a = new FileDialog(this,"Wrterbuch speichern",FileDialog.LOAD);
					a.show();
					if ((filename=a.getFile())!=null){
						dasBuch.writeToFile(filename);
						status.setText("Wrterbuch in"+filename+" geschrieben");
					}
				}catch (Exception e){ new MessageBox("Fehler aufgetreten!","Fehler beim Schreiben der Datei:   "+e,this);}
				}//end if(Wunsaved)
				
			}//end switch
			//case 0 (=OK) wird nicht behandelt und landet damit hier...
		}//end if (W|Unsaved)
		setVisible(false);
    		dispose();
        	System.exit(0);
   	}//end else if (cmd=beenden)
   	
   	
	
  }
}