/** Programm zur Verwaltung von Personen
  * @author Vitus Lorenz-Meyer
  * @version 2.0 vom 26.5.2002:
  	- Elemente nun in binrem Suchbaum
  	- Suchbaum stellt accept fr visitor Interface zur Verfgung (In-,Pre-,Post- und LevelOrder)
  	- BTVisitor Interface definiert 3 Methoden zum visiten eines Baumes, wird immer mit der aktuellen Node aufgerufen, start,end und visitNode
  	- I/O Implementiert
        - serialisiert formatiert nach einer Datei 
        - liest Datenstze aus einer Datei getrennt durch #
        - AdressBuchException hinzugefgt, die Fehler anzeigt und abgefangen werden kann
        - Felder fr Telefonnummer und Hausnummernzusatz zu Person hinzugefgt
        - htmlString() zu jeder Klasse hinzugefgt, gibt einen String formatiert fr HTML Ausgabe
  	
  * history:
  -----------
      version 1.5 vom 1.5 2002:
  	- Klasse Person erweitert um Strae,Hausnummer,PLZ und Ort
  	- Person wirft eine IllegalArgumentException, falls ein Parameter falsch
  	- toString liefert eine Person nun formatiert wie eine Adresse
  	- Elemente nun in einer sortierten linearen Liste gespeichert
  	- Lnge der Liste,sowie Kopf- und Schwanzzeiger in Personen aufgenommen
  	- personAusgeben() liefert alle Personen mit dem gleichen Nachnamen
  	- Interface Adressbuch definiert, wird von Personen implemenetiert
      version 1.0 vom 18.4.2002:
	- erste Version, Person enthlt nur Vor- und Nachname
	- Elemente werden in einer linear, einfach verketteten Liste gespeichert
	- implementiert toString, sowie eine Suche
  */
/**Klasse, die eine einzelne Person reprsentiert */
class Person_voll extends Person{
	protected String strae, wohnort,zusatz,tel;
	protected int plz, hausnummer;
	/** leerer Konstruktor */
	public Person_voll(){
		
		strae=wohnort=zusatz=tel="";
		plz=hausnummer=0;
	}
        
	/** Konstruktor unter Angabe aller Daten 
	*@throws IllegalArgumentException wenn Hausnummer oder PLZ nicht konform sind
	*/
	public Person_voll(String n, String vn, String s, int hau, int p, String w,String z,String t){
		this(n,vn,s,hau,p,w);
		zusatz=z;
		tel=t;
	}
        
        /** Konstruktor ohne Zusatz und Tel */
	public Person_voll(String n, String vn, String s, int hau, int p, String w){
		
		super(n,vn);
		if (hau<0)
			throw new IllegalArgumentException("Hausnummer muss pos. ganze Zahl sein!");
		if (p<0 || p>99999)
			throw new IllegalArgumentException("PLZ muss pos. ganze Zahl <100.000 sein!");
		
		strae=s;
		wohnort=w;
		plz=p;
		hausnummer=hau;
		zusatz=tel="";
		
	}
        
        /** Konstruktor mit Angabe aller Daten in einem Array */
	public Person_voll(String[] a,int i){
		if (i==6){
			name=a[1];
			vorname=a[0];
			strae=a[2];
			hausnummer=(new Integer(a[3])).intValue();
			plz=(new Integer(a[4])).intValue();
			wohnort=a[5];
			zusatz=tel="";
		}
		else if (i==7){
			if (a[6].length() < 4){
				name=a[1];
				vorname=a[0];
				strae=a[2];
				hausnummer=(new Integer(a[3])).intValue();
				plz=(new Integer(a[5])).intValue();
				wohnort=a[6];
				zusatz=a[4];
				tel=""; 
			}else{
				name=a[1];
				vorname=a[0];
				strae=a[2];
				hausnummer=(new Integer(a[3])).intValue();
				plz=(new Integer(a[4])).intValue();
				wohnort=a[5];
				zusatz="";
				tel=a[6];
			}
		}else{
			name=a[1];
			vorname=a[0];
			strae=a[2];
			hausnummer=(new Integer(a[3])).intValue();
			plz=(new Integer(a[5])).intValue();
			wohnort=a[6];			
			zusatz=a[4];
			tel=a[7];  
		}
		
	}
	
	/** liefert einen String in Adressform */
	public String toString(){
		String sep = System.getProperty("line.separator");
		return (String)(vorname + " " + name + sep + strae + " " + hausnummer + " " + zusatz + sep
		+ plz + " " + wohnort + sep + (tel!="" ? ("Telefon: "+tel+sep) : ""));
	}
	
	/** liefert einen String in Adressform in HTML*/
	public String htmlString(){
		String sep = "<br>"+System.getProperty("line.separator");
		return (String)(vorname + " " + name + sep + strae + " " + hausnummer + " " + zusatz + sep
		+ plz + " " + wohnort + sep + (tel!="" ? ("Telefon: "+tel+sep) : ""));
	}
	
}
