/** Programm zur Verwaltung von Personen
  * @author Vitus Lorenz-Meyer
  * @version 2.0 vom 26.5.2002:
  	- Elemente nun in binrem Suchbaum
  	- Suchbaum stellt accept fr visitor Interface zur Verfgung (In-,Pre-,Post- und LevelOrder)
  	- BTVisitor Interface definiert 3 Methoden zum visiten eines Baumes, wird immer mit der aktuellen Node aufgerufen, start,end und visitNode
  	- I/O Implementiert
        - serialisiert formatiert nach einer Datei 
        - liest Datenstze aus einer Datei getrennt durch #
        - AdressBuchException hinzugefgt, die Fehler anzeigt und abgefangen werden kann
        - Felder fr Telefonnummer und Hausnummernzusatz zu Person hinzugefgt
        - htmlString() zu jeder Klasse hinzugefgt, gibt einen String formatiert fr HTML Ausgabe
  	
  * history:
  -----------
      version 1.5 vom 1.5 2002:
  	- Klasse Person erweitert um Strae,Hausnummer,PLZ und Ort
  	- Person wirft eine IllegalArgumentException, falls ein Parameter falsch
  	- toString liefert eine Person nun formatiert wie eine Adresse
  	- Elemente nun in einer sortierten linearen Liste gespeichert
  	- Lnge der Liste,sowie Kopf- und Schwanzzeiger in Personen aufgenommen
  	- personAusgeben() liefert alle Personen mit dem gleichen Nachnamen
  	- Interface Adressbuch definiert, wird von Personen implemenetiert
      version 1.0 vom 18.4.2002:
	- erste Version, Person enthlt nur Vor- und Nachname
	- Elemente werden in einer linear, einfach verketteten Liste gespeichert
	- implementiert toString, sowie eine Suche
  */
/**Klasse, die eine Liste von Personen reprsentiert */
import BTVisitor;
import java.io.*;

class Personen implements Adressbuch{
	public BinaryTree liste;
	protected int laenge;
	/** Konstruktor */
	public Personen(){
		liste = new BinaryTree();
		laenge=0;
	}
	/** Konstruktor mit Angabe vom Tree*/
	public Personen(BinaryTree tr){
		liste = tr;
		laenge=liste.length();
	}
	/** fgt eine Person hinzu
        * @param die Person
        * @throws Adressbuchexception
        */
	public void addPerson(Person_voll per) throws AdressBuchException{
		try{
			liste.insert(((String)per.getName()+","+per.getVorname()),per);
			laenge++;
		}
		catch(RuntimeException e){
			laenge--;
			throw new AdressBuchException("Exception in addPerson->" + e.toString());
			
		}
	}
	/** liefert die Anzahl der Personen im Buch */
	public int anzahl(){
		if (liste.length()!=laenge) System.out.println("this.laenge inkonsisten zu BT.length!!!, correcting");
		return (laenge=liste.length());
	}
	
	/** lscht alle Personen */
	public void personenLoeschen(){
		liste = new BinaryTree();
		laenge=0;
	}
	
        /** lscht eine Person aus der Liste
	* @param String name und Vorname
        */
	public void personLoeschen(String vn, String n) throws AdressBuchException{
		BinaryTree a;
		if (!liste.isIn(n+","+vn)){
			throw new AdressBuchException("personLschen: Person nicht in Liste vorhanden!");
			
		}else{
			try{  a=liste.delete(n+","+vn);
				laenge--;
			}catch(RuntimeException e){
				laenge++;
				throw new AdressBuchException("Exception in personLschen->" + e.toString());
				
			}
			liste=a;
		}
	}
	
	/** guckt ob eine bestimmte Person im Buch ist
	* @param String name und Vorname
        * @return bool
        */
	public boolean enthaeltPerson(String vn, String n){
		return liste.isIn(n+","+vn);
	}
	
        /** liefert Personen aus dem Buch mit einem geg. Nachname
	* @param String name
        * @return wieder ein Adressbuch
        */
	public Adressbuch personAusgeben(String nachname) throws AdressBuchException{
		SearchVisitor v = new SearchVisitor(null,nachname);
		try{
			liste.acceptPre((BTVisitor)v);
		}catch(RuntimeException e){
			throw new AdressBuchException("Exception in personAusgeben->" + e.toString());
		}			
		return new Personen(v.tree);
	}
		
        /** sucht nach einem Muster
	* @param String Muster
        * @return Adressbuch
        */
	public Adressbuch suche(String muster) throws AdressBuchException{
		SearchVisitor v = new SearchVisitor(muster,null);
		try{
			liste.acceptPre((BTVisitor)v);
		}catch(RuntimeException e){
			throw new AdressBuchException("Exception in suche->" + e.toString());
		}return new Personen(v.tree);
	}	
        /** liefert einen String */
	public String toString(){
		return liste.toString();
	}
        /** liefert einen HTML String */
	public String htmlString(){
		return liste.htmlString();
	}
        
        /** schreibt in eine Datei 
        * @param Dateiname
        */
	public void serialize(String filename){
		try {
        		BufferedWriter f2 = new BufferedWriter(new FileWriter(filename));
        		f2.write(this.toString());
         	
       		f2.close();
       
    		} catch (IOException e) {
      			System.out.println("Fehler beim fnen/Erstellen der Datei");
    		}
    	}
    	
        /** liest aus einer Datei 
        * @param Dateiname
        */
	public void readFromFile(String filename) throws AdressBuchException{
    		try {	String line;
    			String[] ar = new String[8];
    			int j,i,k=0;
    			boolean breaky;
			BufferedReader f = new BufferedReader(
           		  new FileReader(filename));
       			while ((line = f.readLine()) != null){
       				j=0;breaky=false;
         			for (i=0; i<8; i++){
         				
         				if (-1 == (j=line.indexOf('#'))){ 
         					if (-1 == (j=line.indexOf('*'))){
         						System.out.println("broke bei"+i+","+k+"teZeile");
         						break;
         					}
         					else if (i<7) breaky=true;
         				}
         				ar[i] = (line.substring(0,j)).trim();
         				line=line.substring(j+1);
         				if (breaky) break;
         				    				
         			}
         			System.out.println("line "+k+" parsed, "+i+" Elements found<br>");
         			addPerson(new Person_voll(ar,i+1));
         			System.out.println("Person " + ar[0]+" read from file and added<br>");		         			         				
       				k++;
       			}
       			f.close();
     		} catch (IOException e) {
       		System.out.println("Fehler beim Lesen der Datei");
		}
	}

}
